﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using RecrownedAthenaeum.SpecialTypes;
using System;

namespace RecrownedAthenaeum.UI.Modular.Modules
{
    /// <summary>
    /// Represents a texture with more information.
    /// </summary>
    public class Image : UIModule, ISpecialDrawable
    {
        /// <summary>
        /// The rotation of the image.
        /// </summary>
        public float rotation = 0f;

        /// <summary>
        /// The texture to be rendered.
        /// </summary>
        public Texture2D Texture { get; set; }
        
        /// <summary>
        /// Scale of of the X axis.
        /// </summary>
        public float ScaleX { get { return (float)bounds.Width / Texture.Width; } set { bounds.Width = (int)(Texture.Width * value); } }

        /// <summary>
        /// Scale of the Y axis.
        /// </summary>
        public float ScaleY { get { return (float)bounds.Height / Texture.Height; } set { bounds.Height = (int)(Texture.Height * value); } }

        /// <summary>
        /// Overall scale.
        /// </summary>
        public float Scale { set { bounds.Height = (int)(Texture.Height * value); bounds.Width = (int)(Texture.Width * value); } }

        /// <summary>
        /// Constructs an image given a texture.
        /// </summary>
        /// <param name="texture">Texture to use.</param>
        public Image(Texture2D texture)
        {
            Texture = texture ?? throw new ArgumentException("Image requires a texture.");
            bounds = texture.Bounds;
        }

        /// <summary>
        /// Draws the image with default values.
        /// </summary>
        /// <param name="batch">The batch to use.</param>
        public override void Draw(SpriteBatch batch)
        {
            batch.Draw(Texture, bounds, null, color, rotation, origin, SpriteEffects.None, 0f);
            base.Draw(batch);
        }

        /// <summary>
        /// Draws the image with more options.
        /// </summary>
        /// <param name="spriteBatch">Batch used.</param>
        /// <param name="destination">Where to draw texture to.</param>
        /// <param name="color">The color tint to use.</param>
        /// <param name="rotation">Rotation of image.</param>
        /// <param name="origin">Origin for the rotation.</param>
        public void Draw(SpriteBatch spriteBatch, Rectangle destination, Color color, float rotation = 0, Vector2 origin = default(Vector2))
        {
            this.color = color;
            this.rotation = rotation;
            this.origin = origin;
            bounds = destination;
            Draw(spriteBatch);
        }
    }
}
